/* Altera FPGAs passive serial loader v0.2 2006/09/25 */
/* stellae (Noel LEMOUEL) / released under GPL */

#include <unistd.h>
#include "spp.h"

/**************************************************/
/************** 'private' functions ***************/
/**************************************************/

static void HAL_sample_data(void);
static unsigned char HAL_check_error(void);

void HAL_init()
{
    /* init low-level access */
    InitParport();
    /* DataBusAsOutput(); */

    /* data port = 0 */
    SetStrobe();                 /* DCLK = 0 */
    SetInitialize();             /* DATA = 0 */
    SetAutofeed();               /* nCONFIG = 1 */

    /* While nCONFIG or nSTATUS are low, the device is in reset. */
    /* TODO check if device is ready */ 
}

void HAL_strobe_nCONFIG()
{
   /* Transition from 0 to 1 on NCONFIG init. device configuration */
   /* look if CONF_DONE = 0 ? */
   
   SetInitialize();      /* nCONFIG = 0 */
   ClearInitialize();    /* nCONFIG = 1 */
   usleep(200);          /* wait 45us */
}

void HAL_sample_data()
{
   SetStrobe();       /* DCLK = 0 */
   ClearStrobe();     /* DCLK = 1 */
}

unsigned char HAL_check_error(void)
{
   /* not implemented */   
   return 0;
}

void HAL_shift_byte(unsigned char rbf_byte)
{
   unsigned char i;

   /* Look for error through NSTATUS every xxKB */
   for(i=0; i<8; i++)
   {
      if ((rbf_byte >> i) & 0x01)
      {
         ClearSelect();            /* DATA = 1 */
      }else{
         SetSelect();              /* DATA = 0 */
    }    
    HAL_sample_data();
   }
}

void HAL_sample_extra_tick(unsigned char cfg_extra_ticks)
{
   unsigned char i;

   SetSelect();                   /* DATA = 0 */
   for(i=0; i<cfg_extra_ticks; i++)
   {
      HAL_sample_data();
   }
}

unsigned char HAL_read_conf_done()
{
   if(!ReadAck())
   {
      return(0x01);
   }else{
      return(0x00);
   }
}

unsigned char HAL_read_init_done()
{
   if(!ReadPaperOut())
   {
      return(0x01);
   }else{
      return(0x00);
   }
}

void HAL_parport_test(void)
{
   while(1)
   {
      SetStrobe();
      sleep(1);
      ClearStrobe();
      sleep(1);
   }
}
