/* "userland" parport abstraction layer (x86), v0.2 2006/09/25 */
/* stellae (Noel LEMOUEL) / released under GPL */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>

#define PARPORT_BASE_ADDR    0x378
#define PARPORT_DATA_REG     PARPORT_BASE_ADDR
#define PARPORT_STATUS_REG   PARPORT_BASE_ADDR+0x01
#define PARPORT_CONTROL_REG  PARPORT_BASE_ADDR+0x02
#define PARPORT_ECR          PARPORT_BASE_ADDR+0x402

/**************************************************/
/************** 'private' functions ***************/
/**************************************************/

static int AllowParportAccess(void);
static void ECPByteModeEnable(void);
static void Reg_Bit_Set(unsigned int _RBS_Reg, unsigned char _RBS_Bit_Num);
static void Reg_Bit_Clear(unsigned int _RBC_Reg, unsigned char _RBC_Bit_Num);
static int Reg_Bit_Read(unsigned int _RBR_Reg, unsigned char _RBR_Bit_Num);

int AllowParportAccess()    /* Allow io acess */ 
{
/* TODO */
/* validation under linux */
/*  if (ioperm(PARPORT_BASE_ADDR,1,1))   { perror("ioperm"); return -1; } */
/*  if (ioperm(PARPORT_DATA_REG,1,1))    { perror("ioperm"); return -1; } */
/*  if (ioperm(PARPORT_CONTROL_REG,1,1)) { perror("ioperm"); return -1; } */

    /* iopl() is used since 0x402 > 0x3ff (ioperm() "limit") */
    if (iopl(3))                         { perror("iopl");   return -1; }
    return 0;   /* No errors */
}

void ECPByteModeEnable()    /* ECP in Byte Mode */
{
    unsigned char zECPBME_swp_byte;
    zECPBME_swp_byte = inb(PARPORT_ECR);
    zECPBME_swp_byte &= 0x1f;
    zECPBME_swp_byte |= 0x20;
    outb(zECPBME_swp_byte, PARPORT_ECR);
}

void Reg_Bit_Set(unsigned int _RBS_Reg, unsigned char _RBS_Bit_Num)
{
    unsigned char zRBS_swp_byte;
    zRBS_swp_byte = inb(_RBS_Reg);
    zRBS_swp_byte |= (0x01 << _RBS_Bit_Num);
    outb(zRBS_swp_byte, _RBS_Reg);
}

void Reg_Bit_Clear(unsigned int _RBC_Reg, unsigned char _RBC_Bit_Num)
{
    unsigned char zRBC_swp_byte;
    zRBC_swp_byte = inb(_RBC_Reg);
    zRBC_swp_byte &= ~(0x01 << _RBC_Bit_Num);
    outb(zRBC_swp_byte, _RBC_Reg);
}

int Reg_Bit_Read(unsigned int _RBR_Reg, unsigned char _RBR_Bit_Num)
{
    unsigned char zRBR_swp_byte;
    zRBR_swp_byte = inb(_RBR_Reg);
    zRBR_swp_byte &= (0x01 << _RBR_Bit_Num);
    return zRBR_swp_byte ? 1 : 0;
}

/**************************************************/
/*************** 'public' functions ***************/
/**************************************************/

void InitParport()
{
    AllowParportAccess();
    ECPByteModeEnable();
}

void DataBusAsOutput()      /* ECP data lines as outputs */
{
    Reg_Bit_Clear(PARPORT_CONTROL_REG,5);
}

void DataBusAsInput()       /* ECP data lines as inputs */
{
    Reg_Bit_Set(PARPORT_CONTROL_REG,5);
}

/* TODO */
/* clean the following 2 functions */
void WriteDataByte(unsigned char _WDB_Byte)
{
   outb(_WDB_Byte, 0x378);
}

unsigned char ReadDataByte()
{
   return inb(PARPORT_DATA_REG);
   /* should return an error if "DataBusAsOutput()" is the current mode */
}

void SetStrobe()
{
    Reg_Bit_Clear(PARPORT_CONTROL_REG,0);   /* hardware inverted */
}

void ClearStrobe()
{
    Reg_Bit_Set(PARPORT_CONTROL_REG,0);     /* hardware inverted */
}

void SetAutofeed()
{
    Reg_Bit_Clear(PARPORT_CONTROL_REG,1);   /* hardware inverted */
}

void ClearAutofeed()
{
    Reg_Bit_Set(PARPORT_CONTROL_REG,1);     /* hardware inverted */
}

void SetInitialize()
{
    Reg_Bit_Set(PARPORT_CONTROL_REG,2);
}

void ClearInitialize()
{
    Reg_Bit_Clear(PARPORT_CONTROL_REG,2);
}

void SetSelect()
{
    Reg_Bit_Clear(PARPORT_CONTROL_REG,3);   /* hardware inverted */
}

void ClearSelect()
{
    Reg_Bit_Set(PARPORT_CONTROL_REG,3);     /* hardware inverted */
}

int ReadAck()
{
    return Reg_Bit_Read(PARPORT_STATUS_REG,6);
}

int ReadBusy()
{
    return Reg_Bit_Read(PARPORT_STATUS_REG,7) ? 0 : 1;   /* hardware inverted */
}

int ReadPaperOut()
{
    return Reg_Bit_Read(PARPORT_STATUS_REG,5);
}

int ReadSelectIn()
{
    return Reg_Bit_Read(PARPORT_STATUS_REG,4);
}

int ReadError()
{
    return Reg_Bit_Read(PARPORT_STATUS_REG,3);
}
